class EmptyQueueException extends Exception {}
class IndexOutOfBoundsException extends Exception {}

public class FIFOQueue<T> {
    private int capacity = 2;
    private Object array[] = new Object[capacity]; 
    private int num = 0; 

    private void makeSpace() {
        capacity *= 2;
        Object temp[] = new Object[capacity];
        for (int i=0; i<num; i++) 
            temp[i] = array[i];

        array = temp;
    }

    /* ---------------------------------------------- */
    /*            PUBLIC INTERFACE METHODS            */
    /* ---------------------------------------------- */

    FIFOQueue() {}
    public void push(T elem) {
        if (num==capacity) makeSpace();
        array[num++] = elem;
    }
    public T pop() throws EmptyQueueException {
        if (num==0) throw new EmptyQueueException();
        T ret = (T)array[0];
        for (int i=1; i<num; i++) array[i-1] = array[i];
        num--;
        return ret;
    }
    public int getNum() { return num; }
    
    public int getCapacity() { return capacity; }

    public T getElem(int i) throws IndexOutOfBoundsException {
        if (i<0 || i>=num) throw new IndexOutOfBoundsException();
        return (T)array[i];
    }

    /* ---------------------------------------------- */

    public static void main(String args[]) {
        FIFOQueue<MyClass> myq = new FIFOQueue<MyClass>();
        FIFOQueue<String> stq = new FIFOQueue<String>();

        for (int i=0; i<5; i++) myq.push(new MyClass());
        for (int i=0; i<5; i++) stq.push("string number " + i);
        
        try {
            while (myq.getNum() > 0 && stq.getNum() > 0) 
                System.out.println("Elem: " + myq.pop().get() + " string: " + stq.pop());
            myq.pop();
        } catch (EmptyQueueException e) {
            System.out.println("Exception: pop() on an empty queue");
        }
    }
}
